<?php

declare(strict_types=1);

namespace Dedi\SyliusMailAdminPlugin\Checker;

use Dedi\SyliusMailAdminPlugin\Exception\UnexpectedMailVariablesException;
use Dedi\SyliusMailAdminPlugin\Form\Registry\MailDefinition;
use Dedi\SyliusMailAdminPlugin\Form\Registry\MailVariableDefinitionRegistryInterface;
use Symfony\Contracts\Translation\TranslatorInterface;

class MailVariablesValidator implements MailVariablesValidatorInterface
{
    private MailVariableDefinitionRegistryInterface $mailVariableDefinitionRegistry;

    private TranslatorInterface $translator;

    public function __construct(
        MailVariableDefinitionRegistryInterface $mailVariableDefinitionRegistry,
        TranslatorInterface $translator
    ) {
        $this->mailVariableDefinitionRegistry = $mailVariableDefinitionRegistry;
        $this->translator = $translator;
    }

    /**
     * @throws UnexpectedMailVariablesException
     */
    public function validateMailVariables(string $mailCode, string $content): void
    {
        $mailVariableDefinitions = $this->mailVariableDefinitionRegistry->getVariableDefinitionByMailCode($mailCode);
        $availableVariables = array_map(static fn (MailDefinition $mailDefinition) => $mailDefinition->getVariableName(), $mailVariableDefinitions);

        preg_match_all('/{{\s*(?<variables>[^{}\s]*)\s*}}/', $content, $matches);

        if (empty($submittedVariables = $matches['variables'])) {
            return;
        }

        if (!empty($unexpectedVariables = array_diff($submittedVariables, $availableVariables))) {
            throw $this->buildException($unexpectedVariables);
        }
    }

    private function buildException(array $unexpectedVariables): UnexpectedMailVariablesException
    {
        return new UnexpectedMailVariablesException(
            $this->translator->trans(
                'dedi_sylius_mail_admin_plugin.unexpected_variables',
                ['%variables%' => implode(', ', $unexpectedVariables)],
                'validators'
            )
        );
    }
}
